// Updates: 2004.09.27, 2003.10.18, 2002.01.08


import java.io.*;
import java.net.*;
import java.util.*;


/**
 * This class can be used to transfer objects through a Network, between 2
 * computers. The first will act as a server whereas the second will be a 
 * client.
 * Important: First load the server, then the client.          
 */
public class Network implements Runnable {
  private Socket s;
  private ObjectInputStream in;
  private ObjectOutputStream out;
  private Vector listeners;
  

  /**
   * Server. 
   */
  public Network(int port) {
    try {
      ServerSocket ss = new ServerSocket(port); 
      s = ss.accept();
    }
    catch (Exception e) {
      System.out.println(e);
    }
    init();
  }


  /**
   * Client. The address is the one of the server.
   */
  public Network(String address, int port) {
    try {
      s = new Socket(address, port);
    }
    catch (Exception e) {
      System.out.println(e);
    }  
    init();    
  }


  /**
   * Adds a new listener which will be notified each time that a new Object
   * is available. 
   */
  public void addNetListener(NetListener listener) {
    listeners.add(listener);
  }
  
  
  /**
   * Removes a listener.
   */
  public void removeNetListener(NetListener listener) {  
    listeners.remove(listener);
  }
  
  
  /**
   * Sends the specified object to the network partner.
   */
  public void send(Object message) {
    try {
      out.writeObject(message);
      out.flush();
    }
    catch (IOException e) {
      System.out.println(e);
    }
  }
 
  
  /** 
   * Closes the connection.
   */
  public void close() {
    try {
      s.close();
    }
    catch (IOException e) {
      System.out.println(e);
    }
  }
  
  
  public void run() { 
    try {
      while(true) fireMessage(in.readObject());
    }
    catch (Exception e) {
      fireMessage(null);
    }
  }
  
  
  private void init() {
    try {
      out = new ObjectOutputStream(s.getOutputStream());
      in = new ObjectInputStream(s.getInputStream());
      listeners = new Vector();
      Thread runner = new Thread(this);
      runner.start();
    }
    catch (Exception e) {
      System.out.println(e);
    }
  }
       
  
  private void fireMessage(Object message) {
    for (int i = 0; i < listeners.size(); i++) {
      ((NetListener)listeners.elementAt(i)).messageArrived(message);
    }
  }
}